// Generated by CoffeeScript 1.6.2
/*
Node Enhanced Interactive Interpreter
=====================================
Provides a simple to use, extensible, embeddable interpreter for
your apps.
*/


(function() {
  var callPluginMethod, nesh, neshInfo, processPlugins,
    __hasProp = {}.hasOwnProperty;

  neshInfo = require('../package');

  /*
  Process a method call on all loaded plugins. This calls the requested
  method on each plugin in the order they were added, skipping plugins
  which do not have the method defined. If the method is defined and it
  takes two parameters (arg, next) then it is assumed to be async and
  treated as such, otherwise treated as a sync call. Callback is
  passed an error if one occurs during processing.
  */


  processPlugins = function(method, arg, callback) {
    var process;

    if (!nesh.plugins.length) {
      return typeof callback === "function" ? callback() : void 0;
    }
    process = function(i) {
      if (i >= nesh.plugins.length) {
        return typeof callback === "function" ? callback() : void 0;
      }
      if (nesh.plugins[i][method] != null) {
        return callPluginMethod(nesh.plugins[i][method], arg, function(err) {
          if (err) {
            return typeof callback === "function" ? callback(err) : void 0;
          }
          return process(i + 1);
        });
      } else {
        return process(i + 1);
      }
    };
    return process(0);
  };

  /*
  Call the possibly async method. callback gets passed an error
  object if an error occurs.
  */


  callPluginMethod = function(method, arg, callback) {
    var err;

    if (method.length === 2) {
      return method(arg, callback);
    } else {
      try {
        method(arg);
      } catch (_error) {
        err = _error;
        return typeof callback === "function" ? callback(err) : void 0;
      }
      return callback();
    }
  };

  nesh = exports;

  nesh.version = neshInfo.version;

  nesh.defaults = {
    prompt: 'nesh> '
  };

  nesh.repl = require('repl');

  nesh.plugins = [];

  nesh.loadLanguage = function(data) {
    switch (typeof data) {
      case 'function':
        return data(nesh);
      case 'string':
        return require("./languages/" + data).setup(nesh);
      default:
        throw new Error("Data must be a function or string! Received " + data);
    }
  };

  nesh.loadPlugin = function(plugin, callback) {
    nesh.plugins.push(plugin);
    if (plugin.setup) {
      return callPluginMethod(plugin.setup, nesh.defaults, callback);
    } else {
      return callback();
    }
  };

  nesh.start = function(opts, callback) {
    var key, value, _ref;

    if (opts == null) {
      opts = {};
    }
    if (typeof opts === 'function') {
      callback = opts;
    }
    _ref = nesh.defaults;
    for (key in _ref) {
      if (!__hasProp.call(_ref, key)) continue;
      value = _ref[key];
      if (opts[key] == null) {
        opts[key] = value;
      }
    }
    return processPlugins('preStart', opts, function(err) {
      var repl;

      if (err) {
        return typeof callback === "function" ? callback(err) : void 0;
      }
      repl = nesh.repl.start(opts);
      return processPlugins('postStart', repl, callback);
    });
  };

  nesh.loadPlugin(require('./plugins/welcome'), function(err) {
    if (err) {
      return console.error('Problem setting up welcome plugin: ', err);
    }
  });

  nesh.loadPlugin(require('./plugins/version'), function(err) {
    if (err) {
      return console.error('Problem setting up version plugin: ', err);
    }
  });

}).call(this);

/*
//@ sourceMappingURL=nesh.map
*/
